// dcf V0.6 170225 qrt@qland.de

#ifndef _DCF_H_
#define _DCF_H_

#include "MicroBit.h"

// DISPLAY_MODE_BLACK_AND_WHITE_LIGHT_SENSE -> 5 + 1 -> 10 ms periods (funny, but oscilloscopes don't lie)
// DISPLAY_MODE_BLACK_AND_WHITE             -> 6 + 1 -> 12 ms
#define TICK_PERIOD_MS      (SYSTEM_TICK_PERIOD_MS + 1)     

#define DCF_INPUT_PIN       MICROBIT_ID_IO_P16      // DCF input pin
#define PIN                 P16                     //

#define PULL                PullUp                  // PullNone or PullUp or PullDown

#define NEGATIVE            0                       // DCF receiver signal logic
#define POSITIVE            1   
#define DCF_LOGIC           POSITIVE        

#define IGNOPU              10                      // ignore pulses after poweron

#define SYNCINI             (1<<0)                  // DCF sync init
#define SYNCSEC             (1<<1)                  //          second
#define SYNC59              (1<<2)                  //          second 59
#define SYNCDAT             (1<<3)                  //          data taken
#define SYNCDAV             (1<<4)                  //               verified
#define SYNCDAR             (1<<5)                  //               ready to take

#define SYNCRES    ~(SYNCSEC | SYNC59 | SYNCDAT | SYNCDAV | SYNCDAR)    // sync restart

class Dcf
{
    public:
        static uint8_t sec;                 // time
        static uint8_t min;
        static uint8_t hou;
        static uint8_t day;
        static uint8_t wday;
        static uint8_t month;
        static uint8_t year;

        static uint8_t status;              // DCF status

        static void init(MicroBit *uBit);
        static void receive();
    
    private:
        static MicroBit *uBit;              // reference

        static uint8_t hsec;                // help time       
        static uint8_t hmin;                
        static uint8_t hhou;
        static uint8_t hday;
        static uint8_t hwday;
        static uint8_t hmonth;
        static uint8_t hyear;
        
        static uint8_t bit;            
        static uint8_t parity;

        static uint32_t dt;

        static void service();
        static void secPulse(MicroBitEvent e);
        static void bitPulse(MicroBitEvent e);
        
        static void incTime();        
        static uint8_t getData();
        static void takeData();
        static uint8_t verifyData();

        static uint8_t decBin(uint8_t b);
        // static uint8_t parity(uint8_t b);
};

#endif