// dcf V0.6 170225 qrt@qland.de

#include "MicroBit.h"
#include "dcf.h"

MicroBit uBit;

#define MODES   4
uint8_t mode;

void onButton(MicroBitEvent e)
{
    if(e.source==MICROBIT_ID_BUTTON_A && e.value==MICROBIT_BUTTON_EVT_CLICK){
        uint8_t m = mode;
        mode = MODES;

        if(++m > MODES-1)
            m = 0;

        uBit.display.stopAnimation();
        uBit.display.print(m==0 ? "T" : (m==1 ? "D" : (m==2 ? "S" : "C")));   
        fiber_sleep(1000);  
        mode = m;               
    }
}

int main()
{
    uBit.init();
    uBit.messageBus.listen(MICROBIT_ID_BUTTON_A, MICROBIT_EVT_ANY, onButton);

    // uBit.display.setDisplayMode(DISPLAY_MODE_BLACK_AND_WHITE_LIGHT_SENSE); 
    // uBit.display.setDisplayMode(DISPLAY_MODE_BLACK_AND_WHITE); 

    Dcf::init(&uBit);    
    create_fiber(Dcf::receive);

    uBit.display.scroll("DCF06");

    char buf[20];

    while(true){
        if(mode == 0){                                  // time, hh:mm:ss
            if(Dcf::sec % 10 == 0){
                sprintf(buf, "%02d:%02d:%02d", Dcf::hou, Dcf::min, Dcf::sec);
                uBit.display.scroll(buf);
            }
        }
        else if(mode == 1){                             // date, DD:MM:YY
            if(Dcf::sec % 10 == 0){
                sprintf(buf, "%02d.%02d.%02d", Dcf::day, Dcf::month, Dcf::year);
                uBit.display.scroll(buf);            
            }
        }
        else if(mode == 2){                             // status
            uint8_t status = Dcf::status & 0b00011111;  // 0 = no signal,       1 = signal,     2 = second 59 detected 
            uint8_t d = 0;                              // 3 = collecting data, 4 = data ready, 5 = data verified

            while(status & 1){                  
                d++;                            
                status >>= 1;
            }

            uBit.display.print(d);
            fiber_sleep(2000);
        }
        else if(mode == 3){                          
            fiber_sleep(1000);            
        }

        fiber_sleep(TICK_PERIOD_MS);
    }
}
